package cz.muni.pa165.race;

import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserService;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;

/**
 * Config for security.
 */
@Configuration
public class SecurityConfig {

  private static final Logger log = LoggerFactory.getLogger("info");

  @Autowired
  OidcUserService userDetailsService;

  /* We were unable to put token for communicattion between microservices.
     In order to start authenticating replace the authorizationHttpRequest with the commented one
   */
  /**
   * Configuration of Spring Security. Sets up OAuth2/OIDC authentication
   * for all URLS except a list of public ones.
   */
  @Bean
  public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
    httpSecurity
        .csrf().disable()
        .authorizeHttpRequests().anyRequest().permitAll();
    /*
    .authorizeHttpRequests(x -> x
        .requestMatchers("/error", "/robots.txt", "/style.css", "/favicon.ico", "/webjars/**")
        .permitAll()
        .anyRequest().authenticated()
    ).oauth2Login(x -> x.successHandler(authenticationSuccessHandler()))
    .logout(x -> x.logoutSuccessUrl("/"));
    */
    return httpSecurity.build();
  }

  /**
   * Handler called when OIDC login successfully completes.
   * It extends the default SavedRequestAwareAuthenticationSuccessHandler
   * that saves the access token
   * to the session.
   * This handler just prints the available info about user to the log and calls
   * its parent implementation.
   *
   * @see SavedRequestAwareAuthenticationSuccessHandler
   */
  @Bean
  public AuthenticationSuccessHandler authenticationSuccessHandler() {
    return new SavedRequestAwareAuthenticationSuccessHandler() {
      @Override
      public void onAuthenticationSuccess(HttpServletRequest req, HttpServletResponse res,
                                          Authentication auth) throws
          ServletException, IOException {
        if (auth instanceof OAuth2AuthenticationToken token
            && token.getPrincipal() instanceof OidcUser user) {
          log.debug("********************************************************");
          log.debug("* user successfully logged in                          *");
          log.debug("********************************************************");
          log.info("user.issuer: {}", user.getIssuer());
          log.info("user.subject: {}", user.getSubject());
          log.info("user.fullName: {}", user.getFullName());
          log.info("user.givenName: {}", user.getGivenName());
          log.info("user.familyName: {}", user.getFamilyName());
          log.info("user.email: {}", user.getEmail());

        }
        super.onAuthenticationSuccess(req, res, auth);
      }
    };
  }
}
