package cz.muni.pa165.race.service;


import cz.muni.pa165.race.data.model.Role;
import cz.muni.pa165.race.data.model.User;
import cz.muni.pa165.race.data.repository.UserRepository;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserRequest;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserService;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Service;


/**
 * TEXT from author
 * This solution solves roles for MANAGER and ENGINEER. As I wrote in discord and got reply
 * that authentication is resolved in oatuth 2.0 scopes for this milestone,
 * I only implemented it for this module. This service will save authenticated user and
 * in the future we can give him permission as a manager or an engineer. I gave example that
 * show that authors of this app are granted manager roles when authenticating.
 */
@Service
public class UserDetailsServiceImpl extends OidcUserService {

  @Autowired
  private UserRepository userRepository;

  private final Set<String> managers = new HashSet<>(Arrays.asList("493356@mail.muni.cz",
      "485386@mail.muni.cz", "49278@mail.muni.cz", "493068@mail.muni.cz"));

  /**
   * Loading user when authenticated.
   *
   * @param userRequest request
   * @return oidc user
   * @throws OAuth2AuthenticationException when error happend
   */
  public OidcUser loadUser(OidcUserRequest userRequest) throws OAuth2AuthenticationException {
    OidcUser oidcUser = super.loadUser(userRequest);

    try {
      return processOidcUser(userRequest, oidcUser);
    } catch (Exception ex) {
      throw new InternalAuthenticationServiceException(ex.getMessage(), ex.getCause());
    }
  }

  private OidcUser processOidcUser(OidcUserRequest userRequest, OidcUser oidcUser) {
    User userInfo = new User();
    userInfo.setEmail(oidcUser.getEmail());

    Optional<User> userOptional = userRepository.findByEmail(userInfo.getEmail());
    if (!userOptional.isPresent()) {
      User user = new User();
      user.setEmail(userInfo.getEmail());
      user.setName(oidcUser.getFullName());

      if (managers.contains(user.getEmail())) {
        user.getRoles().add(Role.MANAGER);
      }
      user.getRoles().add(Role.USER);
      userRepository.save(user);
    }

    return oidcUser;
  }
}
